import PgCursor from 'pg-cursor';
import createLoadQuery from './create-load-query';
import processEventRows from './process-event-rows';
import checkRequestTimeout from './check-request-timeout';
import { DEFAULT_QUERY_TIMEOUT, MINIMAL_QUERY_TIMEOUT } from './constants';
import { isConnectionTerminatedError, makeConnectionError, makeKnownError } from './errors';

const getEventLoaderNative = async (pool, filter) => {
  var _checkRequestTimeout;

  const {
    cursor,
    eventTypes,
    aggregateIds
  } = filter;
  const sqlQuery = createLoadQuery(pool, {
    cursor,
    eventTypes,
    aggregateIds
  });

  if (sqlQuery === null) {
    throw new Error(`Invalid event filter: ${JSON.stringify(filter)}`);
  }

  const vacantTimeInMillis = (_checkRequestTimeout = checkRequestTimeout(pool)) !== null && _checkRequestTimeout !== void 0 ? _checkRequestTimeout : DEFAULT_QUERY_TIMEOUT;
  const statementTimeout = Math.max(vacantTimeInMillis, MINIMAL_QUERY_TIMEOUT);
  let currentCursor = cursor;
  const client = new pool.Postgres({
    keepAlive: false,
    connectionTimeoutMillis: statementTimeout,
    statement_timeout: statementTimeout,
    ...pool.connectionOptions
  });
  let connectionTerminated = false;
  client.on('error', error => {
    if (isConnectionTerminatedError(error)) {
      connectionTerminated = true;
    }

    return;
  });
  let pgCursor;

  try {
    await client.connect();
  } catch (error) {
    throw makeConnectionError(error);
  }

  try {
    pgCursor = client.query(new PgCursor(sqlQuery));
  } catch (error) {
    var _makeKnownError;

    throw (_makeKnownError = makeKnownError(error)) !== null && _makeKnownError !== void 0 ? _makeKnownError : error;
  }

  return {
    async close() {
      // await pgCursor.close() // may never resolve due to https://github.com/brianc/node-postgres/issues/2642
      // client.end is enough anyway since cursor lives as long as the connection
      // client end may still hang if connection terminated and pgcursor exists
      if (!connectionTerminated) {
        await client.end();
      }
    },

    async loadEvents(limit) {
      try {
        const rows = await pgCursor.read(limit);
        const result = processEventRows(pool, currentCursor, rows);
        currentCursor = result.cursor;
        return result;
      } catch (error) {
        var _makeKnownError2;

        if (isConnectionTerminatedError(error)) {
          connectionTerminated = true;
        }

        throw (_makeKnownError2 = makeKnownError(error)) !== null && _makeKnownError2 !== void 0 ? _makeKnownError2 : error;
      }
    },

    get cursor() {
      return currentCursor;
    },

    isNative: true
  };
};

export default getEventLoaderNative;
//# sourceMappingURL=get-event-loader-native.js.map