"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _assert = _interopRequireDefault(require("assert"));

var _eventstoreBase = require("@resolve-js/eventstore-base");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const describe = async (pool, options) => {
  var _options$estimateCoun;

  const {
    executeStatement,
    secretsTableName,
    escapeId,
    escape,
    eventsTableName,
    databaseName
  } = pool;
  const databaseNameAsId = escapeId(databaseName);
  const eventsTableNameAsId = escapeId(eventsTableName);
  const secretsTableNameAsId = escapeId(secretsTableName);
  const freezeTableName = `${eventsTableName}-freeze`;
  const threadsTableAsId = escapeId(`${eventsTableName}-threads`);
  let cursor = undefined;

  if (options && options.calculateCursor) {
    const threads = await executeStatement(`SELECT "threadId", "threadCounter" FROM ${databaseNameAsId}.${threadsTableAsId} ORDER BY "threadId" ASC`);
    const threadArray = threads.map(row => {
      return +row.threadCounter;
    });
    cursor = (0, _eventstoreBase.threadArrayToCursor)(threadArray);
  }

  const estimateCounts = (_options$estimateCoun = options === null || options === void 0 ? void 0 : options.estimateCounts) !== null && _options$estimateCoun !== void 0 ? _options$estimateCoun : false;
  const rows = await executeStatement(`SELECT
    ${estimateCounts ? `(SELECT reltuples::bigint FROM pg_class WHERE oid = ${escape(databaseNameAsId + '.' + eventsTableNameAsId)}::regclass)` : `(SELECT COUNT(*) FROM ${databaseNameAsId}.${eventsTableNameAsId})`} AS "eventCount",
    ${estimateCounts ? `(SELECT reltuples::bigint FROM pg_class WHERE oid = ${escape(databaseNameAsId + '.' + secretsTableNameAsId)}::regclass)` : `(SELECT COUNT(*) FROM ${databaseNameAsId}.${secretsTableNameAsId})`} AS "secretCount",
    (SELECT COUNT(*) FROM ${databaseNameAsId}.${secretsTableNameAsId} WHERE "secret" IS NOT NULL) AS "setSecretCount",
    (SELECT COUNT(*) FROM ${databaseNameAsId}.${secretsTableNameAsId} WHERE "secret" IS NULL) AS "deletedSecretCount", 
    (SELECT "timestamp" FROM ${databaseNameAsId}.${eventsTableNameAsId} ORDER BY "timestamp" DESC LIMIT 1) AS "lastEventTimestamp",
    (SELECT EXISTS (SELECT FROM "information_schema"."tables" WHERE "table_schema" = ${escape(databaseName)} AND "table_name" = ${escape(freezeTableName)})) AS "isFrozen"
    `);

  _assert.default.strictEqual(rows.length, 1, 'Wrong number of description rows');

  const row = rows[0];
  return {
    eventCount: +row.eventCount,
    secretCount: +row.secretCount,
    setSecretCount: +row.setSecretCount,
    deletedSecretCount: +row.deletedSecretCount,
    lastEventTimestamp: +row.lastEventTimestamp,
    isFrozen: !!row.isFrozen,
    cursor: cursor,
    resourceNames: {
      eventsTableName,
      databaseName
    }
  };
};

var _default = describe;
exports.default = _default;
//# sourceMappingURL=describe.js.map